﻿using System;

using Sandbox;
using Sandbox.Common;
using Sandbox.Common.Components;
using Sandbox.Common.ObjectBuilders;
using Sandbox.Definitions;
using Sandbox.Engine;
using Sandbox.Game;
//Copyright (c) 2015 Joseph W. Barker
//MIT License
using Sandbox.Game.Entities;
using Sandbox.Game.Entities.Cube;
using Sandbox.Game.World;
using Sandbox.ModAPI;
using Ingame=Sandbox.ModAPI.Ingame;

namespace InterGridComms
{
    [MyEntityComponentDescriptor(typeof(MyObjectBuilder_CubeGrid))]
    public class BlockTracker : MyGameLogicComponent
    {
        private MyObjectBuilder_EntityBase m_objectBuilder;

        public override void Init(MyObjectBuilder_EntityBase objectBuilder)
        {
            m_objectBuilder=objectBuilder;
            
            ((IMyCubeGrid)Entity).OnBlockAdded += CubeGrid_OnBlockAdded;
            ((IMyCubeGrid)Entity).OnBlockRemoved += CubeGrid_OnBlockRemoved;
        }

        public override MyObjectBuilder_EntityBase GetObjectBuilder(bool copy)
        {
            return m_objectBuilder;
        }

        public override void Close()
        {
            ((IMyCubeGrid)Entity).OnBlockAdded -= CubeGrid_OnBlockAdded;
            ((IMyCubeGrid)Entity).OnBlockRemoved -= CubeGrid_OnBlockRemoved;
            if(IGC.Instance!=null) IGC.Instance.UnregisterGrid((IMyCubeGrid)Entity); //Just to be sure
            base.Close();
        }

        public static void CubeGrid_OnBlockAdded(IMySlimBlock sb)
        {
            var blk=sb.FatBlock;
            if(blk==null) return;
            if(!(blk is Ingame.IMyRadioAntenna) && !(blk is IMyLaserAntenna) && !(blk is Ingame.IMyBeacon)) return;
            if(IGC.Instance!=null) IGC.Instance.RegisterBlock((IMyTerminalBlock)blk);
        }

        public static void CubeGrid_OnBlockRemoved(IMySlimBlock sb)
        {
            var blk=sb.FatBlock;
            if(blk==null) return;
            if(!(blk is Ingame.IMyRadioAntenna) && !(blk is IMyLaserAntenna) && !(blk is Ingame.IMyBeacon)) return;
            if(IGC.Instance!=null) IGC.Instance.UnregisterBlock((IMyTerminalBlock)blk);
        }
    }
}
