//Copyright (c) 2015 Joseph W. Barker
//MIT License
using System;
using System.Collections.Generic;

namespace InterGridComms
{
    public class PrioritySequence<T>
    {
        public PrioritySequence()
        {
            LowestPrio=long.MaxValue;
            HighestPrio=long.MinValue;
        }

        public T Next()
        {
            if(lvls.Count==0) throw new System.InvalidOperationException("No elements to remove");

            HashSet<T> lvl=lvls[LowestPrio];
            var en=lvl.GetEnumerator();
            en.MoveNext();
            T obj=en.Current;

            Remove(LowestPrio,obj,lvl);

            return obj;
        }

        public void Add(long prio, T obj)
        {
            HashSet<T> lvl;
            if(!lvls.TryGetValue(prio,out lvl))
            {
                if(cached.Count>0)
                    lvl=cached.Pop();
                else
                    lvl=new HashSet<T>();
                lvls.Add(prio,lvl);

                if(prio<LowestPrio) LowestPrio=prio;
                if(prio>HighestPrio) HighestPrio=prio;
            }

            lvl.Add(obj);
        }

        public void Remove(long prio, T obj)
        {
            HashSet<T> lvl;
            if(!lvls.TryGetValue(prio,out lvl))
                return;
            Remove(prio,obj,lvl);
        }

        private void Remove(long prio, T obj, HashSet<T> lvl)
        {
            lvl.Remove(obj);
            if(lvl.Count==0)
            {
                lvls.Remove(prio);
                cached.Push(lvl);
                if(cached.Count > lvls.Count && cached.Count > 1)
                    cached.Clear();

                if(lvls.Count==0)
                {
                    LowestPrio=long.MaxValue;
                    HighestPrio=long.MinValue;
                }
                else if(prio==LowestPrio)
                    //TODO: Could be slow if priorities are sparse, maybe add heap to track lowest?
                    while(!lvls.TryGetValue(++LowestPrio,out lvl));
            }
        }
        
        public void Clear()
        {
            lvls.Clear();
            cached.Clear();
            LowestPrio=long.MaxValue;
            HighestPrio=long.MinValue;
        }

        //public void Update(long oldPrio, T obj, long newPrio);

        public long LowestPrio { get; private set; }
        public long HighestPrio { get; private set; }

        private Dictionary<long,HashSet<T>> lvls=new Dictionary<long,HashSet<T>>();
        private Stack<HashSet<T>> cached=new Stack<HashSet<T>>();
    }
}
